@@ -19,6 +19,23 @@ soup_address_family_get_type (void)
 	return etype;
 }
 
+/* enumerations from "soup-cookie-jar.h" */
+GType
+soup_cookie_jar_accept_policy_get_type (void)
+{
+	static GType etype = 0;
+	if (G_UNLIKELY (etype == 0)) {
+		static const GEnumValue values[] = {
+			{ SOUP_COOKIE_JAR_ACCEPT_ALWAYS, "SOUP_COOKIE_JAR_ACCEPT_ALWAYS", "always" },
+			{ SOUP_COOKIE_JAR_ACCEPT_NEVER, "SOUP_COOKIE_JAR_ACCEPT_NEVER", "never" },
+			{ SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY, "SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY", "no-third-party" },
+			{ 0, NULL, NULL }
+		};
+		etype = g_enum_register_static (g_intern_static_string ("SoupCookieJarAcceptPolicy"), values);
+	}
+	return etype;
+}
+
 /* enumerations from "soup-date.h" */
 GType
 soup_date_format_get_type (void)
